#!/usr/bin/ruby
# vim:foldmethod=marker
require 'net/http'
require 'openssl/ssl'
require 'pp'
require 'serialport.so'
require 'rubygems'
require 'httpclient'
require 'rss/2.0'
require 'open-uri'
require 'yaml'
require 'uri'

port_str = "/dev/ttyUSB0"  #may be different for you
baud_rate = 9600
data_bits = 8
stop_bits = 1
parity = SerialPort::NONE

class LuntBuildStatus # {{{
	def initialize()
	end

	def check(url)
		content = ""
		open(url) do |s| content = s.read end
		content.split(/\<tr/).each { |line|
			if line =~ /images\/failed\.gif/
				project = ""
				line.split(/\r/).each { |innerline|
					unless(innerline =~ /\</ || innerline =~ /\>/)
						project = innerline
						break
					end
				}
				raise Exception.new(project + " Luntbuild")
			end
		}
	end
end # }}}

class CruisecontrolStatus # {{{
	def initialize()
	end

	def check(url)
		#puts "checking " + url
		content = "" # raw content of rss feed will be loaded here
		open(url) do |s| content = s.read end
		rss = RSS::Parser.parse(content, false)
		if rss.items[0].description == "Build FAILED!"
			raise Exception.new(rss.items[0].title)
		end
	end
end # }}}

class CruisecontrolRBStatus # {{{
	def initialize()
	end

	def check(url, username, password)
		#puts "checking " + url
		content = "" # raw content of rss feed will be loaded here
        client = HTTPClient.new
		client.ssl_config.verify_mode = OpenSSL::SSL::VERIFY_NONE
		if username && password
			client.www_auth.set_auth(URI.parse(url), username, password)
		end
		content = client.get_content(url)
		#open(url) do |s| content = s.read end
		rss = RSS::Parser.parse(content, false)
		rss.items.each { |item|
			if item.title.include?("failed")
				raise Exception.new(item.title)
			end
		}
	end
end # }}}

# Main {{{
if __FILE__ == $0
	SerialPort.open(port_str, baud_rate, data_bits, stop_bits, parity) { |sp|
		begin
			configfile = "ambient_build.yaml"
			if !(FileTest.exist?(configfile))
				configfile = ENV["HOME"] + "/.ambient_build"
				if !(FileTest.exist?(configfile))
					configfile = "/etc/ambient_build.yaml"
					if !(FileTest.exist?(configfile))
						raise Exception.new("Requires a config file in one of the following: ./ambient_build.yaml, $HOME/.ambient_build, /etc/ambient_build.yaml")
					end
				end
			end
			config = YAML.load_file(configfile)

			config.each_key { |key|
				case config[key]["type"]
					when "lunt"
						lunt_status = LuntBuildStatus.new()
						config[key]["projects"].each { |project|
							lunt_status.check(project["url"]) if project.has_key?("url")
						}
					when "cruisecontrol"
						cc_status = CruisecontrolStatus.new()
						config[key]["projects"].each { | project|
							cc_status.check(project["url"]) if project.has_key?("url")
						}
					when "ccrb"
						ccrb_status = CruisecontrolRBStatus.new()

						config[key]["projects"].each { | project|
							username = NIL
							username = project["username"] if project.has_key?("username")
							password = NIL
							password = project["password"] if project.has_key?("password")
							ccrb_status.check(project["url"], username, password) if project.has_key?("url")
						}
					else
						raise Exception.new("Unknown builder: " + config[key]["type"])
				end
			}
			sp.puts("roam")
		rescue SocketError
			$stderr.print "Socket Error: " + $!.to_s + "\n"
			sp.puts("%0000FF")
		rescue Errno::ENETUNREACH
			$stderr.print "Network Unreachable Error: " + $!.to_s + "\n"
			sp.puts("%0000FF")
		rescue Timeout::Error
			$stderr.print "Timeout Error: " + $!.to_s + "\n"
			sp.puts("%00AAFF")
		rescue Exception
			$stderr.print "Build failed: " + $!.to_s + "\n"
			sp.puts("%FF0000")
		end
	}
end
#}}}


