#!/usr/bin/python

"""
Copyright (C) 2008 Devon Jones <soulcatcher@evilsoft.org>

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along
with this library; if not, write to the Free Software Foundation, Inc., 59
Temple Place, Suite 330, Boston, MA  02111-1307  USA

For questions regarding this application contact
Devon Jones <soulcatcher@evilsoft.org>
http://www.evilsoft.org/lightbiff/
"""

import sys
import os
import imaplib
import time
import serial
from email.utils import parsedate

SERIAL_PORT = "/dev/ttyUSB0"
BAUD = 9600
SERVER = "IMAPSERVER"
PORT = 993
USER = "USERNAME"
PASSWORD = "PASSWORD"
SECONDS = 3600
REDSECONDS = 1800
UPDATE = 20
SSL = True

def get_imap_handle(imapconfig):
	try:
		imap = None
		if SSL:
			imap = imaplib.IMAP4_SSL(imapconfig['server'])
		else:
			imap = imaplib.IMAP4(imapconfig['server'])
		imap.login(imapconfig['login'], imapconfig['password'])
	except Exception, e:
		print("IMAP on fire eh?")
		exit(-1)
	return imap

def get_oldest_unseen_date(imap):
	typ, data = imap.search(None, 'UNSEEN')
	oldestmsg = None
	oldtime = None
	idlist = data[0].split()
	if len(idlist) > 0:
		num = idlist[0]
		typ, data = imap.fetch(num, '(RFC822)')
		msgdate = get_date(data[0][1])
		return msgdate

def get_date(message):
	lines = message.splitlines()
	datestr = None
	for line in lines:
		if line.startswith("Date: "):
			datestr = line.split(": ")[1]
			msgdate = time.mktime(parsedate(datestr))
			return msgdate

def check_time(ser, msgdate):
	now = time.time()
	if not ser.isOpen():
		ser.open()

	if msgdate:
		if (now - msgdate) > (SECONDS + REDSECONDS):
			ser.write("%FF0000")
		elif (now - msgdate) > SECONDS:
			ser.write("#FF0000")
		else:
			percent = (now - msgdate) / SECONDS
			value = 512 * percent
			if value > 256:
				byte1 = 255
				byte2 = 255 - int(value - 256)
			else:
				byte1 = int(value)
				byte2 = 255
			color = "#" + hex_chars(byte1) + hex_chars(byte2) + "00"
			ser.write(color)
	else:
		ser.write("roam")

def hex_chars(num):
	value = str(hex(num)).upper()[2:4]
	if len(value) == 1:
		return "0" + value
	return value

def main():
	ser = serial.Serial(SERIAL_PORT, BAUD, timeout=1)
	while 0 == 0:
		config = {'login': USER, 'server': SERVER, 'port': PORT, 'password': PASSWORD}
		imap = get_imap_handle(config)
		imap.select()
		msgdate = get_oldest_unseen_date(imap)
		check_time(ser, msgdate)
		imap.close()
		imap.logout()
		time.sleep(UPDATE)

if __name__ == "__main__":
	main()
